import pathlib
import typing

import event_stream
import numpy
import scipy.ndimage


def apply(
    input: pathlib.Path,
    output: pathlib.Path,
    begin: int,
    end: int,
    parameters: dict[str, typing.Any],
) -> None:
    with event_stream.Decoder(input) as decoder:
        count = numpy.zeros((decoder.width, decoder.height), dtype=numpy.uint64)
        for packet in decoder:
            if packet["t"][-1] < begin:
                continue
            if packet["t"][0] >= end:
                break
            if packet["t"][0] < begin and packet["t"][-1] >= end:
                events = packet[
                    numpy.logical_and(packet["t"] >= begin, packet["t"] < end)
                ]
            elif begin is not None and packet["t"][0] < begin:
                events = packet[packet["t"] >= begin]
            elif end is not None and packet["t"][-1] >= end:
                events = packet[packet["t"] < end]
            else:
                events = packet
            numpy.add.at(count, (events["x"], events["y"]), 1)
        shifted: list[numpy.ndarray] = []
        for x, y in ((1, 0), (0, 1), (1, 2), (2, 1)):
            kernel = numpy.zeros((3, 3))
            kernel[x, y] = 1.0
            shifted.append(
                scipy.ndimage.convolve(
                    input=count,
                    weights=kernel,
                    mode="constant",
                    cval=0.0,
                )
            )
        ratios = numpy.divide(count, numpy.maximum.reduce(shifted) + 1.0)
        mask = ratios < parameters["ratio"]
        with event_stream.Decoder(input) as decoder:
            with event_stream.Encoder(
                output,
                "dvs",
                decoder.width,
                decoder.height,
            ) as encoder:
                for packet in decoder:
                    if packet["t"][-1] < begin:
                        continue
                    if packet["t"][0] >= end:
                        break
                    if packet["t"][0] < begin and packet["t"][-1] >= end:
                        events = packet[
                            numpy.logical_and(packet["t"] >= begin, packet["t"] < end)
                        ]
                    elif begin is not None and packet["t"][0] < begin:
                        events = packet[packet["t"] >= begin]
                    elif end is not None and packet["t"][-1] >= end:
                        events = packet[packet["t"] < end]
                    else:
                        events = packet
                    events = events[mask[events["x"], events["y"]]]
                    if len(events) > 0:
                        encoder.write(events)



import argparse
import os
import pathlib
import re
import sys

ANSI_COLORS_ENABLED = os.getenv("ANSI_COLORS_DISABLED") is None

TIMECODE_PATTERN = re.compile(r"^(\d+):(\d+):(\d+)(?:\.(\d+))?$")


def with_suffix(path: pathlib.Path, suffix: str):
    return path.parent / f"{path.name}{suffix}"


def format_bold(message: str) -> str:
    if ANSI_COLORS_ENABLED:
        return f"\033[1m{message}\033[0m"
    return message


def info(icon: str, message: str):
    sys.stdout.write(f"{icon} {message}\n")
    sys.stdout.flush()


def error(message: str):
    sys.stderr.write(f"❌ {message}\n")
    sys.exit(1)


def timestamp_to_timecode(timestamp: int):
    hours = timestamp // (60 * 60 * 1000000)
    timestamp -= hours * 60 * 60 * 1000000
    minutes = timestamp // (60 * 1000000)
    timestamp -= minutes * 60 * 1000000
    seconds = timestamp // 1000000
    timestamp -= seconds * 1000000
    return f"{hours:02d}:{minutes:02d}:{seconds:02d}.{timestamp:06d}"


def timestamp_to_short_timecode(timestamp: int):
    hours = timestamp // (60 * 60 * 1000000)
    timestamp -= hours * 60 * 60 * 1000000
    minutes = timestamp // (60 * 1000000)
    timestamp -= minutes * 60 * 1000000
    seconds = timestamp // 1000000
    timestamp -= seconds * 1000000
    timestamp_string = "" if timestamp == 0 else f".{timestamp:06d}".rstrip("0")
    if hours > 0:
        return f"{hours}:{minutes:02d}:{seconds:02d}{timestamp_string}"
    if minutes > 0:
        return f"{minutes}:{seconds:02d}{timestamp_string}"
    return f"{seconds}{timestamp_string}"


def timecode(value: str) -> int:
    if value.isdigit():
        return int(value)
    match = TIMECODE_PATTERN.match(value)
    if match is None:
        raise argparse.ArgumentTypeError(
            f"expected an integer or a timecode (12:34:56.789000), got {value}"
        )
    result = (
        int(match[1]) * 3600000000 + int(match[2]) * 60000000 + int(match[3]) * 1000000
    )
    if match[4] is not None:
        fraction_string: str = match[4]
        if len(fraction_string) == 6:
            result += int(fraction_string)
        elif len(fraction_string) < 6:
            result += int(fraction_string + "0" * (6 - len(fraction_string)))
        else:
            result += round(float("0." + fraction_string) * 1e6)
    return result



name_to_icon = {
    "ant": "🐜",
    "badger": "🦡",
    "bat": "🦇",
    "bear": "🐻",
    "beaver": "🦫",
    "beetle": "🪲",
    "bird": "🐦",
    "bison": "🦬",
    "blowfish": "🐡",
    "boar": "🐗",
    "buffalo": "🐃",
    "bug": "🐛",
    "butterfly": "🦋",
    "camel": "🐪",
    "cat": "🐈",
    "chicken": "🐔",
    "chipmunk": "🐿 ",
    "coral": "🪸",
    "cow": "🐄",
    "crab": "🦀",
    "cricket": "🦗",
    "crocodile": "🐊",
    "deer": "🦌",
    "dodo": "🦤",
    "dog": "🐕",
    "dolphin": "🐬",
    "dove": "🕊 ",
    "dragon": "🐉",
    "duck": "🦆",
    "eagle": "🦅",
    "elephant": "🐘",
    "ewe": "🐑",
    "fish": "🐟",
    "flamingo": "🦩",
    "fly": "🪰",
    "fox": "🦊",
    "frog": "🐸",
    "giraffe": "🦒",
    "goat": "🐐",
    "gorilla": "🦍",
    "hamster": "🐹",
    "hedgehog": "🦔",
    "hippopotamus": "🦛",
    "honeybee": "🐝",
    "horse": "🐎",
    "kangaroo": "🦘",
    "koala": "🐨",
    "leopard": "🐆",
    "lion": "🦁",
    "lizard": "🦎",
    "llama": "🦙",
    "lobster": "🦞",
    "mammoth": "🦣",
    "monkey": "🐒",
    "mosquito": "🦟",
    "mouse": "🐁",
    "octopus": "🐙",
    "orangutan": "🦧",
    "otter": "🦦",
    "owl": "🦉",
    "ox": "🐂",
    "panda": "🐼",
    "parrot": "🦜",
    "peacock": "🦚",
    "penguin": "🐧",
    "pig": "🐖",
    "poodle": "🐩",
    "rabbit": "🐇",
    "raccoon": "🦝",
    "ram": "🐏",
    "rat": "🐀",
    "rhinoceros": "🦏",
    "rooster": "🐓",
    "sauropod": "🦕",
    "scorpion": "🦂",
    "seal": "🦭",
    "shark": "🦈",
    "shrimp": "🦐",
    "skunk": "🦨",
    "sloth": "🦥",
    "snail": "🐌",
    "snake": "🐍",
    "spider": "🕷 ",
    "squid": "🦑",
    "swan": "🦢",
    "t-rex": "🦖",
    "tiger": "🐅",
    "turkey": "🦃",
    "turtle": "🐢",
    "unicorn": "🦄",
    "whale": "🐋",
    "wolf": "🐺",
    "zebra": "🦓",
}


def composite_name_to_icon(composite_name: str):
    for name, icon in name_to_icon.items():
        if name in composite_name:
            return icon
    return "🐾"


def generate_names(count: int):
    import copy

    import coolname
    import coolname.data

    config = copy.copy(coolname.data.config)
    config["animal"]["words"] = list(name_to_icon.keys())
    config["subj"]["lists"] = ["animal"]
    generator = coolname.RandomGenerator(config)
    coolname_length = 4
    for length in (2, 3):
        if count < generator.get_combinations_count(length) / 2:
            coolname_length = length
            break
    names = []
    while len(set(names)) < count:
        while len(names) < count:
            names.append(generator.generate_slug(coolname_length))
    names.sort()
    return names
